(in-package :LISA-USER) ; nu merge din fisier
(defconstant *max-time* 100)

(defparameter *install-root-path* cg-user::*install-root-path*)
(defparameter *rules-path* cg-user::*rules-path*)
(defparameter *tdl-path* cg-user::*tdl-path*)
(defparameter *theory-path* cg-user::*theory-path*)


(setf *strict-facts* nil)  ;;lista ce contine toate faptele stricte
(setf *defeasible-facts* nil) ;;lista c econtienfaptele defeasible
(setf *time* 5)

(clear)
(reset)

(defun empty-file (file)
  (with-open-file (output-stream *rules-path*
                 :direction :output
		 :if-exists :supersede)
    (format output-stream "")))

(empty-file *rules-path*)

(defun member-fact (f lis)
"It verifies if the fact f is member in a list (test is made by equal)"
  (do ((l lis (cdr l)) (rez nil))
      ((endp l) rez)
      (if (equal (car f) (car (car l))) (setf rez T))))

(defun concatenate-symbol (&rest symbols)
  (let ( (*print-case* (readtable-case *readtable*)) )
    (declare (special *print-case* *readtable*))
    (intern (format nil "~{~A~}" symbols))))

(defun time-pattern (init final formula)
  (loop for i from init to final by formula collect i))


(defun q (&key (runp t))
  ;(reset) ; reset sterge faptele
  ;(walk)
   (format t "Strict facts ~S~%" *strict-facts* )
   (format t "Defeasible facts ~S~%" *defeasible-facts*)
   (load *tdl-path*)
   (setf *strict-facts* nil)  ;;lista ce contine toate faptele stricte
   (setf *defeasible-facts* nil) ;;lista c econtienfaptele defeasible 
   (run))

(defun defeasible-facts-names ()
  "car *defeasible-facts* intoarce lista cu celule cons (Poz . Nume)
  Pentru extragerrea doar a numelui se mai aplica si cdr"
  (mapcar #'(lambda (x) (cdr (car x))) *defeasible-facts*))

(defun s ()
  (q)
  (format t "Generating dynamics rules")
  (do ((names (defeasible-facts-names) (cdr names)))
        ((endp names))
       (cond ((not (symbolp (car names))) (eval (car names) )))); daca numele nu reprezinta simboluri 
                                                                ;(ci liste) atunci se evalueaza
   (format t "Loadind nested rules...")  
  (load *rules-path*) ;se incarca fisierul cu noile reguli create
   (run)
  )

(defun all-facts (x y)
  "Union of two lists (test is with equal)"
  (cond ((endp x) y)
        ((member (car x) y :test #'equal) (all-facts (cdr x) y))
        (T (cons (car x) (all-facts (cdr x) y))))) 

(defun all-about (x)
  "Return all information about fact x available in the 
 global lists *strict-facts* and *defeasible-facts*"
  (do ((aux  (all-facts *strict-facts* *defeasible-facts*) (cdr aux))
     (rez nil))
     ((endp aux) rez)
     (cond ((equal x (car (car aux))) (setf rez (car aux))))))
  
(defun is-already-derived (name t-issue t-maturity)
    (cond ((endp (all-about name)) nil)
          ((and (= t-issue (nth 2 (all-about name)))
                (= t-maturity (nth 3 (all-about name)))) t)
          (T nil)))


(defun is-strict (q)
"Return true if the fact x has certain factor cf = 1
 (all-about x) will return (x 1.0)"
   (equal (nth 1 (all-about q)) 1.0))

;(;defun +D (q)
;"Return true if the fact x has certain factor cf = 1
; (all-about x) will return (x 1.0)"
;   (equal (nth 1 (all-about q)) 1.0))

(defun is-defeasible (q)
 "Return true if the fact x has certain factor cf < 1
 (all-about x) will return nil only when the fact is nor defeasible or strict
  therefore the fact was already asserted and it is not strict"
  (and (not (equal (all-about q) nil)) 
       (not (is-strict q))))

(defun strength-of (q)
 "Return cf of fact x
  (all-about x) will return (x 1.0)" 
   (nth 1 (all-about q)))

(defun is-strict-fact (fapt)
  (= (lisa:cf fapt) 1.0))

(defun is-strict-rule (supportive-rule)
  (= (lisa:cf supportive-rule) 1.0))

(defun add-strict-fact (name t-issue t-maturity)
  (setf *strict-facts* (append (list (list name 1.0 t-issue t-maturity)) *strict-facts*))
  (format t "[~S] Strict conclusion: ~A[~S,~S] (~,3F) ~%" (rule) name t-issue t-maturity 1.0))

(defun add-defeasible-fact (name cf t-issue t-maturity)
  (setf *defeasible-facts* (append (list (list name cf t-issue t-maturity)) *defeasible-facts*))
  (format t "[~S] Defeasible conclusion: ~A [~S,~S] (~,3F)~%" (rule) name t-issue t-maturity cf))

(defun generate-interval (a b)
  (do ((i a (+ i 1))(rez nil))
      ((> i b) (reverse rez))
      (setf rez (cons i rez))))

;;intoarce true daca intersectia nu este lista vida
;; <=> exista un moment de timp comun
(defun is-valid (x y x1 y1)
   (not (endp (intersection (generate-interval x y)
                     (generate-interval x1 y1)))))
(defun +D (poz q time) 
  (let* ((info (all-about (cons poz q))) 
       (t1 (nth 2 info))
       (t2 (nth 3 info))
       (cf (nth 1 info))) 
  (if (equal 1.0 cf) (<= t1 time t2))))
 
(defun +def (poz q time) 
  (let* ((info (all-about (cons poz q))) 
       (t1 (nth 2 info))
       (t2 (nth 3 info))
       (cf (nth 1 info))) 
  (if (and (not (equal 1.0 cf))
          (not (equal info nil))) (<= t1 time t2))))
 
(defun +def+D (poz q time) ; nu stiu daca +def mai este folosita undeva cu sensul faptele ddefeasible si care nu sunt strict derivable
  (let* ((info (all-about (cons poz q))) 
       (t1 (nth 2 info))
       (t2 (nth 3 info))
       (cf (nth 1 info))) 
  (if (and ;(not (equal 1.0 cf))
          (not (equal info nil))) (<= t1 time t2))))

;(defun +def+D (poz q time) ; nu stiu daca +def mai este folosita undeva cu sensul faptele ddefeasible si care nu sunt strict derivable
;  (do* ((info (all-about (cons poz q))) 
;       (t1 (nth 2 info))
;       (t2 (nth 3 info))
;       (cf (nth 1 info))) 
;  (if (and ;(not (equal 1.0 cf))
;          (not (equal info nil))) (<= t1 time t2))))


(defun cf (poz q)
  (let* ((info (all-about (cons poz q))) 
       (cf (nth 1 info)))
  (if (not (endp info)) cf)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(deftemplate fapt ()
   (slot name)
   (slot t-issue)
   (slot t-maturity))
   
(deftemplate supportive-rule ()
   (slot name)
   (slot premise)
   (slot premise-time-issue)
   (slot premise-time-maturity)
   (slot conclusion)
   (slot conclusion-time-issue)
   (slot conclusion-time-maturity))

(deftemplate defeater ()
   (slot name)
   (slot premise)
   (slot premise-time-issue) 
   (slot premise-time-maturity)
   (slot conclusion)
   (slot conclusion-time-issue)
   (slot conclusion-time-maturity))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
(defun fact (label name1 cf1 t-issue t-maturity)
(let ((?name-car label) 
      (?name-cdr name1) 
      (?cf cf1) 
      (?t-issue t-issue) 
      (?t-maturity t-maturity))
(assert (fapt (name (cons ?name-car ?name-cdr)) 
              (t-issue ?t-issue)
              (t-maturity ?t-maturity)) :cf ?cf)
(if (not (= ?cf 1.00)) (defeasible (concatenate-symbol ?name-cdr '[ ?t-issue  t-maturity '] ?cf  ) ?cf '(poz alwaystrue) 0 *max-time* 
                                   ?name-car ?name-cdr ?t-issue ?t-maturity)))) ;;pentr a crea o regula care sustine faptul initial)

(defun fact-pattern (label name cf-list tissue-list tmaturity-list)
"Pnetru fiecare elemente corespondente din tissue-list si tmaturitu-list se 
  genereaza fapte"
  (setq label-list (copy-list cf-list))
    (setq name-list (copy-list cf-list))
    (fill label-list label); vor fi 5 liste de aceeasi dimensiune pentru a putea aplica mapcar
    (fill name-list name) 
    (format t "Generating facts from the pattern:")
    (print label-list)
    (print name-list)
    (print cf-list)
    (print tissue-list)
    (print tmaturity-list)
    (mapcar #'fact label-list name-list cf-list tissue-list tmaturity-list))
  
  

(defun defeat (name1 cf1 label-p p x2 y2 label-c c u1 v1)
  (let* ((?name name1) 
       (?cf cf1) 
       (?premise-car label-p) 
       (?premise-cdr p) 
       (?x2 x2) 
       (?y2 y2)
       (?conclusion-car label-c) 
       (?conclusion-cdr c) 
       (?u1 u1) 
       (?v1 v1)
       (?premise (cons ?premise-car ?premise-cdr))
       (?conclusion (cons ?conclusion-car ?conclusion-cdr)))
  (assert (defeater (name ?name) 
                    (premise ?premise) 
                    (premise-time-issue ?x2) 
                    (premise-time-maturity ?y2)
                    (conclusion ?conclusion)
                    (conclusion-time-issue ?u1)
                    (conclusion-time-maturity ?v1)) :cf ?cf)))


(defun defeat-pattern (name cf-list labels premises x1-list y1-list label-c c u-list v-list)

    (setq name-list (copy-list cf-list)) ;creez copii ale listelor deoarece fill are efect destructiv
    (setq [-list (copy-list cf-list)) 
    (setq ]-list (copy-list cf-list)) 
    (setq labels-list (copy-list cf-list))
    (setq premises-list (copy-list cf-list))
    (setq label-c-list (copy-list cf-list))
    (setq c-list (copy-list cf-list))
    
    (fill name-list name)
    (fill [-list '[)
    (fill ]-list '])
    (setf name-list 
          (mapcar #'concatenate-symbol name-list [-list x1-list y1-list u-list v-list ]-list))   ; fiecare regula trebuie sa aiba nume unic
    (format t "The unique defeater name is: ~S~%" name-list)
    (fill labels-list labels)
    (fill premises-list premises)
    (fill label-c-list label-c)
    (fill c-list c) 
    (format t "Generating rules from the defeater-pattern:~%")
    (print name-list)
    (print cf-list)
    (print labels-list)
    (print premises-list)
    (print x1-list)
    (print y1-list)
    (print u-list)
    (print v-list)
    (mapcar #'defeat name-list cf-list labels-list premises-list 
                         x1-list y1-list 
                         label-c-list c-list u-list v-list))



(defun strict (name1 premises x1 y1 label-c c u v)
"name1 -numele regulii
 premises = lista de premise de forma '(poz a) (momentan exista o singura premisa)
 x1 = t issue pentru premisa
 y1 = t maturity pentru premisa
 label-c = poz/neg concluzia
 c = concluzia
 u  = t issue pentru concluzie
 v = t maturity pentru concluzie" 
    (defeasible name1 1.00 premises x1 y1 label-c c u v))

(defun strict-pattern (name premises x1-list y1-list label-c c u-list v-list)
  (setq cf-list (copy-list x1-list)); crez o lista de aceasi lungime cu listele sablon
  (fill cf-list 1.00)
  (defeasible-pattern name cf-list premises x1-list y1-list label-c c u-list v-list))

(defun defeasible (name1 cf1 premises x1 y1 label-c c u v)
  (let* ((?name name1) 
        (?cf cf1) 
        (?premise-car (car premises))
        (?premise-cdr (nth 1 premises)) 
        (?x1 x1) 
        (?y1 y1)
        (?conclusion-car label-c)
        (?conclusion-cdr c)
        (?u u) 
        (?v v)
        (?premise (cons ?premise-car ?premise-cdr))
        (?conclusion (cons ?conclusion-car ?conclusion-cdr)))
    (setf rule `(defrule ,?name (:cf ,?cf) 
                   =>
                  (assert (supportive-rule (name ,?name) 
                                           (premise ',?premise) 
                                           (premise-time-issue ',?x1) 
                                           (premise-time-maturity ',?y1) 
                                           (conclusion ',?conclusion) 
                                           (conclusion-time-issue ',?u)
                                           (conclusion-time-maturity ',?v)) :cf ,?cf )
                  (format t "[~S] Asserting supporting-rule ~S~%" (rule) ',?name) ))
    (with-open-file (output-stream *rules-path*
        		             :direction :output
		                     :if-exists :append)
	          (format output-stream "~S~%" rule))))

(defun defeasible-pattern (name cf-list premises x1-list y1-list label-c c u-list v-list)

    (setq name-list (copy-list cf-list)) ;creez copii ale listelor deoarece fill are efect destructiv
    (setq [-list (copy-list cf-list)) 
    (setq ]-list (copy-list cf-list)) 
    (setq premises-list (copy-list cf-list))
    (setq label-c-list (copy-list cf-list))
    (setq c-list (copy-list cf-list))
    
    (fill name-list name)
    (fill [-list '[)
    (fill ]-list '])
    (setf name-list 
          (mapcar #'concatenate-symbol name-list [-list x1-list y1-list u-list v-list ]-list))   ; fiecare regula trebuie sa aiba nume unic
    (format t "The unique name is: ~S~%" name-list)
    (fill premises-list premises)
    (fill label-c-list label-c)
    (fill c-list c) 
    (format t "Generating rules from the supporting rules pattern:~%")
    (print name-list)
    (print cf-list)
    (print premises-list)
    (print x1-list)
    (print y1-list)
    (print u-list)
    (print v-list)
    (mapcar #'defeasible name-list cf-list premises-list 
                         x1-list y1-list 
                         label-c-list c-list u-list v-list))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(fact 'poz 'alwaystrue 1.0 0 *max-time*)
(load *theory-path*)
(load *rules-path*)
(format t "The rules were generated~%")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 

;; Orice fapt cu cf=1 este considerat strict derivabil

(defrule +D1 (:salience -10)  
  (?f (fapt (name ?name) 
            (t-issue ?t-issue) 
            (t-maturity ?t-maturity)))
  (test (and (is-strict-fact ?f)
             ;(not (is-strict ?name))
             )) ;;nu a fost derivat inca
        =>  
  (add-strict-fact ?name 
                   ?t-issue 
                   ?t-maturity))

(defrule +D2 (:salience -20)  
  (?p (fapt (name ?name) 
            (t-issue ?x) 
            (t-maturity ?y)))
  (?r (supportive-rule (premise ?name) 
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)  
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u) 
                       (conclusion-time-maturity ?v)))
  (test (and (is-strict-fact ?p)  ;premisa este stricta si 
             (is-strict-rule ?r) ;regula este stricta
             ;(not (is-strict ?conclusion)) ;nu a fost derivata deja
             (is-valid ?x ?y ?x1 ?y1))); exista un moment de timp comun intre fapt si premisa regulii suport
                                   ; moment la car eregula se poate activa 
   =>  
   (add-strict-fact ?conclusion 
                    (max ?x ?u) ; concluzia devene valida doar cand premisa se activeaza
                    ?v) 
   (assert (fapt (name ?conclusion) 
           (t-issue (max ?x ?u))
           (t-maturity ?v)) :cf 1.0))
 
;; A strict fact (cf=1) is also a defeasible one
(defrule +def1.1 (:salience -30)    
  (?f (fapt (name ?name)
            (t-issue ?u)
            (t-maturity ?v)) )
  (test (and (is-strict-fact ?f) ;fie e stricta
             ;(not (is-defeasible ?name))
             )) ;;nu este deja in lista
   =>  
   (add-defeasible-fact ?name 
                        1.0 
                        ?u 
                        ?v))
 
;; O concluzie q cu (0 < cf < 1) 
;; care are premisa valida
;; si pentru care nu exista un defeater cu concluzia q este considerat defeasible
(defrule +def2 (:salience -40)
  (?p (fapt (name ?name)
            (t-issue ?x)
            (t-maturity ?y)))
  (?r (supportive-rule (name ?rulename)
                       (premise ?name)
                       (premise-time-issue ?x1)
                       (premise-time-maturity ?y1) 
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
  (not (defeater (conclusion ?conclusion)))
  (test (and ;(not (is-defeasible ?conclusion))  ;sa nu fie derivata deja
             (or (not (is-strict-fact ?p)) ; cazul acesta este tratat de D2
                 (not (is-strict-rule ?r)))
             (is-valid ?x ?y ?x1 ?y1)))
    =>	
    (let ((cf (* (lisa:cf ?p) (lisa:cf ?r))))
    (add-defeasible-fact ?conclusion cf 
                         (max ?x ?u) 
                         ?v)
    (assert (fapt (name ?conclusion) 
                  (t-issue (max ?x ?u))
                  (t-maturity ?v)) :cf cf)
    (format t "~S[~S,~S](~S) because: ~S[~S,~S](~S) ^ rule ~S ^ no defeater with conclusion ~S~%" 
           ?conclusion (max ?x ?u) ?v cf ?name ?x ?y (lisa:cf ?p) ?rulename ?conclusion)))	


;;  Un fapt q cu (0 < cf < 1) pentru care exista un 
;;  defeater cu concluzia q, 
;;  dar a carui premisa nu poate fi derivata 
;;  este considerat defeasible
(defrule +def3.1.1 (:salience -50)
  (?p (fapt (name ?name)
            (t-issue ?x)
            (t-maturity ?y)))
  (?r (supportive-rule (name ?rulename)
                       (premise ?name) 
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)
                       (conclusion ?conclusion)
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
  (?d (defeater (name ?defeatername)
                (premise ?premise) 
                (premise-time-issue ?x2)
                (premise-time-maturity ?y2)
                (conclusion ?conclusion)))
  (not (fapt (name ?premise) 
             (t-issue ?xx)
             (t-maturity ?yy)))
  (test (and (is-valid ?x ?y ?x1 ?y1) ;premisa regulii suport este valida
              ;(not (is-defeasible ?conclusion) ) ; concluzia nu a fost derivata deja
              (or (not (is-strict-fact ?p)) ; cazul acesta este tratat de D2
                  (not (is-strict-rule ?r)))))
    =>	
   (let ((cf (* (lisa:cf ?p) (lisa:cf ?r))))
        (add-defeasible-fact ?conclusion 
                             cf 
                             (max ?x ?u) 
                             ?v)
        (assert (fapt (name ?conclusion)
                      (t-issue (max ?x ?u))
                      (t-maturity ?v)) :cf cf)
        (format t "~S[~S,~S](~S) because: ~S(~S) ^ rule ~S ^ defeater ~S 
           with conclusion ~S has no a valid premise ~S.~%" 
           ?conclusion (max ?x ?u) ?v cf ?name (lisa:cf ?p) 
           ?rulename ?defeatername ?conclusion ?premise)))

;; O concluzie q pentru care exista o regula suport
;; a carei premisa e derivabila
;; si pentru care exista un defeater
;; cu premisa derivabila dar nu e valida la monetul potrivit
;; pentru a activa defeaterul
(defrule +def3.1.2 (:salience -55)
  (?p (fapt (name ?name)
            (t-issue ?x)
            (t-maturity ?y)))
  (?r (supportive-rule (name ?rulename)
                       (premise ?name) 
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
  (?d (defeater (name ?defeatername)
                (premise ?premise)
                (premise-time-issue ?x2)
                (premise-time-maturity ?y2)
                (conclusion ?conclusion)))
  (fapt (name ?premise) 
        (t-issue ?xx)
        (t-maturity ?yy))
  (test (and (is-valid ?x ?y ?x1 ?y1) ;premisa regulii suport este valida
              ;(not (is-defeasible ?conclusion) ) ; concluzia nu a fost derivata deja
              (or (not (is-strict-fact ?p)) ; cazul acesta este tratat de D2
                  (not (is-strict-rule ?r)))
              (not (is-valid ?xx ?yy ?x2 ?y2))));premisa defeaterului nu e valida
  =>	
 (let ((cf (* (lisa:cf ?p) (lisa:cf ?r))))
   (add-defeasible-fact ?conclusion 
                        cf 
                        (max ?x ?u) 
                        ?v)
   (assert (fapt (name ?conclusion)
                 (t-issue (max ?x ?u))
                 (t-maturity ?v)) :cf cf)
   (format t "~S[~S,~S](~S) because: ~S(~S) ^ rule ~S ^ defeater ~S 
           with conclusion ~S has premise ~S, but it is not valide within [~S,~S].~%" 
           ?conclusion (max ?x ?u) ?v cf ?name (lisa:cf ?p) ?rulename 
           ?defeatername ?conclusion ?premise ?x1 ?y1)))

;; O concluzie q pentru care exista o regula suport
;; a carei premisa e derivabila
;; si pentru care exista un defeater
;; cu premisa derivabila dar a carui concluzie nu e valida
;; la momentul potrivit pentru a bloca conditia initiala
(defrule +def3.2.1 (:salience -60)
  (?p (fapt (name ?name)
            (t-issue ?x)
            (t-maturity ?y) ))
  (?r (supportive-rule (name ?rulename)
                       (premise ?name) 
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
  (?d (defeater (name ?defeatername)
                (premise ?premise)
                (premise-time-issue ?x2)
                (premise-time-maturity ?y2)
                (conclusion ?conclusion)
                (conclusion-time-issue ?u1)
                (conclusion-time-maturity ?v1)))
  (fapt (name ?premise) 
        (t-issue ?xx)
        (t-maturity ?yy))
  (test (and (is-valid ?x ?y ?x1 ?y1) ;premisa regulii suport este valida
              ;(not (is-defeasible ?conclusion) ) ; concluzia nu a fost derivata deja
              (or (not (is-strict-fact ?p)) ; cazul acesta este tratat de D2
                  (not (is-strict-rule ?r)))
              (is-valid ?xx ?yy ?x2 ?y2) ;premisa defeaterului e valida
              (not (is-valid ?u ?v ?u1 ?v1)))); concluzia defeaterului nu e valida
  =>	
 (let ((cf (* (lisa:cf ?p) (lisa:cf ?r))))
   (add-defeasible-fact ?conclusion 
                        cf 
                        (max ?x ?u) 
                        ?v)
   (assert (fapt (name ?conclusion)
                 (t-issue (max ?x ?u))
                 (t-maturity ?v)) :cf cf)
   (format t "~S[~S,~S](~S) 
           because: premise ~S[~S,~S](~S)
           ^ rule ~S 
           ^ defeater ~S 
           with conclusion ~S[~S,~S] is not valide within [~S,~S].~%" 
           ?conclusion (max ?x ?u) ?v cf 
           ?name ?x ?y (lisa:cf ?p) 
           ?rulename 
           ?defeatername 
           ?conclusion ?u1 ?v1 (max ?x ?u) ?v)))

;;  Un fapt q cu (0 < cf < 1) pentru care exista un 
;;  defeater cu concluzia q si a carui premisa poate fi derivata 
;;  si a carui concluzie este valida
;;este considerat defeasible doar daca regula care il sustine este
;;  mai puternica decat defeaterul
(defrule +def3.2.2 (:salience -65)
  (?p (fapt (name ?name)
            (t-issue ?x)
            (t-maturity ?y)))
  (?r (supportive-rule (name ?rulename)
                       (premise ?name) 
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
  (?d (defeater (name ?defeatername)
                (premise ?premise)
                (premise-time-issue ?x2)
                (premise-time-maturity ?y2)
                (conclusion ?conclusion)
                (conclusion-time-issue ?u1)
                (conclusion-time-maturity ?v1)))
  (?pd (fapt (name ?premise) 
        (t-issue ?xx)
        (t-maturity ?yy)))
  (test (and (is-valid ?x ?y ?x1 ?y1) ;premisa regulii suport este valida
              ;(not (is-defeasible ?conclusion) ) ; concluzia nu a fost derivata deja
              (or (not (is-strict-fact ?p)) ; cazul acesta este tratat de D2
                  (not (is-strict-rule ?r)))
              (is-valid ?xx ?yy ?x2 ?y2);premisa defeaterului e valida
              (is-valid ?u ?v ?u1 ?v1) ;concluzia defeaterului este valida
             (> (* (lisa:cf ?p) (lisa:cf ?r)) ; defeaterul e mai slab
                (* (lisa:cf ?pd) (lisa:cf ?d)))))
  =>	
 (let ((cf (* (lisa:cf ?p) (lisa:cf ?r))))
   (add-defeasible-fact ?conclusion 
                        cf 
                        (max ?x ?u) 
                        ?v)
   (assert (fapt (name ?conclusion)
                 (t-issue (max ?x ?u))
                 (t-maturity ?v)) :cf cf)
   (format t "~S[~S,~S](~S) 
           because: ~S[~S,~S](~S) 
           ^ rule ~S ^ 
           defeater ~S 
           with conclusion ~S is weaker.~%"
           ?conclusion (max ?x ?u) ?v cf 
           ?name ?x ?y (lisa:cf ?p) 
           ?rulename 
           ?defeatername 
           ?conclusion)))


;(defun +d-nested-premise (?name time)
;  "Este premisa imbricata derivabila la momentul t"
;(+def+D (nth 0 (car (cdr (nth 2 (car (cdr (cdr ?name))))))) ;intoarce 'poz (eticheta premisei imbricate)                           'b ;(cdr (nth 2 (cdr ?name))) 
;        (nth 1 (car (cdr (nth 2 (car (cdr (cdr ?name))))))) ; intoarce 'a numele premisei imbricate
;        time))

(defun +def+D-or-interval (label name ti tm)
  "Este premisa imbricata derivabila la un moment t in itnerval [ti tm]"
  (format t "Testing the validity of the sentence ~S ~S in  one instance of time between ~S and ~S....~%" label name ti tm)
  (do ((time ti (+ time 1))
       (rez nil))
      ((> time tm) rez)
      (if (+def+D label name time) (setf rez T))
      (format t "The result until time [~S].......~S~%" time rez)))  
  
(defun +def+D-and-interval (label name ti tm)
  "Este concluzia imbricata derivabila la toate momentele t in itnerval [ti tm]"
  (format t "Testing the validity of the sentence ~S ~S in each instance of time between ~S and ~S....~%" (eval label) (eval name) ti tm)
  (format t "Defeasible-facts ~S~%" *defeasible-facts*)
  (do ((time ti (+ time 1))
       (rez t))
      ((> time tm) rez)
      (if (not (+def+D (eval label) (eval name) time)) (setf rez nil)); am un apostrof care deranjeaza compararea
      (format t "The result until time [~S]......~S~%" time rez)))  

(defun is-valid-rule (name)
 "?name=(poz qoute (strict 'R41 '(POZ A) 1 25 'POZ 'B 9 9)))!!!are 3 elemente
  intoarce tru daca exista relatia a->b"
 (format t "Now testing the validity of the nested rule: ~S~%" name)
  (do* ((rule (nth 2 name)) ;doar regula imbricata
      (nested-premise-label (nth 0 (nth 1 (nth 2 rule)))) 
      (nested-premise-name (nth 1 (nth 1 (nth 2 rule))))
      (nested-premise-tissue (nth 3 rule))
      (nested-premise-tmaturity (nth 4 rule))     
      (nested-conclusion-label (nth 5 rule))
      (nested-conclusion-name (nth 6 rule))
      (nested-conclusion-tissue (nth 7 rule))
      (nested-conclusion-tmaturity (nth 8 rule))
      (rez nil)
      (out nil))
      
      ((equal out T) rez)
      (cond  ((or ;premisa imbricata e derivabila la un moment
              (not (+def+D-or-interval nested-premise-label
                               nested-premise-name 
                               nested-premise-tissue 
                               nested-premise-tmaturity))          
                  ;concluzia e derivabila la toate monetele cerute
              (+def+D-and-interval nested-conclusion-label 
                                      nested-conclusion-name 
                                      nested-conclusion-tissue 
                                      nested-conclusion-tmaturity))
                        (setf out T)
                        (setf rez T))
                       
              (T (setf out T)
                 (format t "The rule is not valid because the premise ~S ~S [~S,~S] is not valid, 
                                   and the conclusion ~S ~S [~S,~S] is valid ( not p V q).~%" 
                               nested-premise-label
                               nested-premise-name 
                               nested-premise-tissue 
                               nested-premise-tmaturity
                               nested-conclusion-label 
                               nested-conclusion-name 
                               nested-conclusion-tissue 
                               nested-conclusion-tmaturity)))
);do
   )  
  
(defrule premises (:salience 100)
  
 (?r (supportive-rule (name ?rulename)
                       (premise ?name) ;?name=(poz qoute (strict 'R41 '(POZ A) 1 25 'POZ 'B 9 9)))!!!are 3 elemente
                       (premise-time-issue ?x1) 
                       (premise-time-maturity ?y1)
                       (conclusion ?conclusion) 
                       (conclusion-time-issue ?u)
                       (conclusion-time-maturity ?v)))
(test (and (not (symbolp (cdr ?name))) ;premisa este o regula
          (is-valid-rule ?name)));            
  =>     (assert (fapt (name ?name)
                       (t-issue ?x1) 
                       (t-maturity ?y1)))
         (format t "Nested premise: ~S~%" ?name))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;














