;;; Code for the form named :tdl of class dialog.
;;; The inspector may add event-handler starter code here,
;;; and you may add other code for this form as well.
;;; The code for recreating the window is auto-generated into 
;;; the *.bil file having the same name as this *.cl file.

(in-package :common-graphics-user)

(defparameter *setup-file* "C:\\tdl.setup")
(defparameter *install-root-path* "E\\lisp\\tdl-0.1.1") 

(defun read-init-file ()
(with-open-file (in *setup-file*) 
  (setf *install-root-path* (namestring (read-line in)))))

(read-init-file)

(defparameter *os* 'windows)

(defun make-tdl-path (relative-path)
  (concatenate 'string (namestring *install-root-path*)
               relative-path))

(defparameter *theory-path* (make-tdl-path "\\lisa\\temp.tdl"))
(defparameter *lisa-path* (make-tdl-path "\\lisa\\install.lisp"))
(defparameter *tdl-path* (make-tdl-path "\\lisa\\w.lisp"))
(defparameter *rules-path* (make-tdl-path "\\lisa\\rules.lisp"))


(defparameter *windows-test-list* '("\\test\\1-facts.tdl" 
                                    "\\test\\2-strict-rules.tdl"
                                    "\\test\\3-defeasible-rules.tdl"
                                    "\\test\\4-defeaters.tdl"
                                    "\\test\\5-patterns-facts.tdl"
                                    "\\test\\6-patterns-strict.tdl"
                                    "\\test\\7-patterns-defeasible.tdl"
                                    "\\test\\8-patterns-defeaters.tdl"
                                    "\\test\\9-dynamic-rules.tdl"
                                    "\\test\\10-hidden-rules.tdl"
                                    ))

(defparameter *linux-test-list* '("/test/1-facts.tdl" 
                                    "/test/2-strict-rules.tdl"))

(defun choose (os)
  (cond ((equal os 'windows) (defparameter *test-list* *windows-test-list*)) 
         (T (defparameter *test-list* *linux-test-list*))))                 

(choose *os*)


(load *lisa-path* :print t)
(load *tdl-path* :print t)


(defclass tdl (non-refreshing-window)
  ())

(defclass defeasible-theory (multi-line-lisp-text)
  ())

;************** buttons *****************************
(defun tdl-button-new-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  (when new-value
    (new-theory (parent widget)))
  t) ; Accept the new value

(defun tdl-button-open-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  (when new-value
    (open-theory (parent widget)))
  t) ; Accept the new value


(defun tdl-button-save-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  (when new-value
    (save-theory (parent widget)))
  t) 

(defun tdl-button-load-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  (when new-value
    (load-theory (parent widget)))
      t) 

(defun tdl-button-run-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  ;(in-package lisa-user)
  ;(import 'lisa-user::s)
  (when new-value
      (format t "Iesire ~S~%" (lisa-user::s)))
      t) 

(defun tdl-button-query-on-change (widget new-value old-value)
  (declare (ignore-if-unused widget new-value old-value))
  (when new-value
    (format t "~S" (eval (value (find-component :lisp-text-query (find-window :TDL)))))
    (setf (value (find-component :static-text-answer (find-window :TDL))) 
           (eval (value (find-component :lisp-text-query (find-window :TDL)))))     
       )  
      t) 

;************menu***************************************
(defun new-theory-menu (widget)
  (new-theory (parent widget))
  t)

(defun open-theory-menu (widget)
  (open-theory (parent widget))
  t)

(defun save-theory-menu (widget)
  (save-theory (parent widget))
  t)

(defun load-theory-menu (widget)
  (load-theory (parent widget))
  t)

(defun help-about (widget)
  t)

(defun help-how-to (widget)
  t)

;*********** toolbar *************************************

(defun toolbar-multi-picture-button-2-on-change (widget
                                                 new-value
                                                 old-value)
  (declare (ignore-if-unused widget new-value old-value))
  ;; Do the action only when a button is being pressed (not unpressed)
  (when new-value
    (when new-value
      (let ((tdl (parent widget)))
        (case (first new-value)
          (:new
           (new-theory tdl))
          (:open
           (open-theory tdl))
          (:save 
           (save-theory tdl))
          (:load 
           (load-theory tdl))
         (t nil))))
   (not new-value))
    )


;********************** functions ***************************
(defun new-theory (pane &optional path)
    (setf (value (find-component :multi-line-lisp-text-theory (find-window :TDL))) ""))


(defun open-theory (pane &optional path)
  "Reads a defeasible theory file and displays it in the multiline lisp editor."
  (unless path
    (setq path (ask-user-for-existing-pathname
                "Defeasible Theory File to Open"
                :allowed-types '(("Defeasible Theory Files" . "*.tdl")))))
  (unless path
    (window-message pane "Opening CANCELED.")
    (return-from open-theory))
  
  (with-open-file (in path)
    (do ((l (read-line in) (read-line in nil 'eof))
         (theory (find-component :multi-line-lisp-text-theory (find-window :TDL)))
         (file (read-text path)))
        ((eq l 'eof) "Reached end of file.")
         (setf (value theory) file)
      )))


(defun save-theory (pane &optional path)
  "Saves a defeasible theory."
   (unless path
    (setq path (ask-user-for-new-pathname
                "Defeasible Theory File to Save"
                :allowed-types '(("Defeasible Theory Files" . "*.tdl")))))
  (unless path
    (window-message pane "Saving CANCELED.")
    (return-from save-theory))
  (with-open-file (out path
                         :direction :output
                         :if-exists :supersede)
     (write-line (value (find-component :multi-line-lisp-text-theory (find-window :TDL))) out)
     (format t "~S" (value (find-component :multi-line-lisp-text-theory (find-window :TDL))))

  )) 

(defun load-theory (pane &optional path)
  (unless path
    (setq path *theory-path*))
  (save-theory pane path)
  (remove-first-and-last-line path)
  (load path :print t)
)

(defun remove-first-and-last-line (path)
  (with-open-file (in path)
    (do ((l (read-line in) (read-line in nil 'eof))
         (all-file nil))
        ((eq l 'eof) (remove (car (last all-file)) (remove (car all-file) all-file)))
         (setf all-file (append all-file (list l)))
      )))



(defun tdl-multi-item-list-test-on-double-click (dialog widget)
  (declare (ignore-if-unused dialog widget))
  (read-init-file)
  (let ((tdl (parent widget))
        (test (value (find-component :multi-item-list-test (find-window :tdl)))))
    (format t "~S" test)
    (case (car test)
      (:1-facts (open-theory tdl (make-tdl-path (nth 0 *test-list*))))
      (:2-strict-rules (open-theory tdl (make-tdl-path (nth 1 *test-list*))))
      (:3-defeasible-rules (open-theory tdl (make-tdl-path (nth 2 *test-list*))))
      (:4-defeaters (open-theory tdl (make-tdl-path (nth 3 *test-list*))))
      (:5-patterns-facts (open-theory tdl (make-tdl-path (nth 4 *test-list*))))
      (:6-patterns-strict (open-theory tdl (make-tdl-path (nth 5 *test-list*))))
      (:7-patterns-defeasible (open-theory tdl (make-tdl-path (nth 6 *test-list*))))
      (:8-patterns-defeaters (open-theory tdl (make-tdl-path (nth 7 *test-list*))))
      (:9-dynamic-rules (open-theory tdl (make-tdl-path (nth 8 *test-list*))))
      (:10-hidden-rules (open-theory tdl (make-tdl-path (nth 9 *test-list*))))
      (t 'Eroare)))
  t)
